@@ -18,15 +18,7 @@ pub fn generate<T: Service>(service: &T, proto_path: &str) -> TokenStream {
         /// Generated client implementations.
         #[allow(dead_code, unused_imports)]
         pub mod #client_mod {
-            use hrpc::{
-                IntoRequest, Request,
-                client::{
-                    Client, HttpClient,
-                    error::ClientResult,
-                    socket::Socket
-                },
-                exports::http::Uri,
-            };
+            use hrpc::client::prelude::*;
 
             #service_doc
             #[derive(Debug, Clone)]
@@ -35,26 +27,29 @@ pub fn generate<T: Service>(service: &T, proto_path: &str) -> TokenStream {
             }
 
             impl #service_ident {
-                pub fn new(host_url: Uri) -> ClientResult<Self> {
+                /// Create a new client.
+                pub fn new<U>(host: U) -> ClientResult<Self>
+                where
+                    U: TryInto<Uri>,
+                    U::Error: Debug,
+                {
                     Ok(Self {
-                        inner: Client::new(host_url)?,
+                        inner: Client::new(host.try_into().expect("invalid URL"))?,
                     })
                 }
 
+                /// Create a new client using an already created hRPC generic client.
                 pub fn new_inner(inner: Client) -> Self {
                     Self { inner }
                 }
 
+                /// Create a new client using an hRPC HTTP client and host URL.
                 pub fn new_http(http: HttpClient, host_url: Uri) -> ClientResult<Self> {
                     Ok(Self {
                         inner: Client::new_inner(http, host_url)?,
                     })
                 }
 
-                pub fn inner(&self) -> &Client {
-                    &self.inner
-                }
-
                 #methods
             }
         }
@@ -96,11 +91,11 @@ fn generate_unary<T: Method>(method: &T, proto_path: &str, path: String) -> Toke
     let (request, response) = method.request_response_name(proto_path);
 
     quote! {
-        pub async fn #ident(
-            &mut self,
-            request: impl IntoRequest<#request>,
-        ) -> ClientResult<#response> {
-            self.inner.execute_request(#path, request.into_request()).await
+        pub async fn #ident<Req>(&mut self, req: Req) -> ClientResult<#response>
+        where
+            Req: IntoRequest<#request>,
+        {
+            self.inner.execute_request(#path, req.into_request()).await
         }
     }
 }
@@ -110,11 +105,11 @@ fn generate_streaming<T: Method>(method: &T, proto_path: &str, path: String) ->
     let (request, response) = method.request_response_name(proto_path);
 
     quote! {
-        pub async fn #ident(
-            &mut self,
-            request: impl IntoRequest<()>,
-        ) -> ClientResult<Socket<#request, #response>> {
-            self.inner.connect_socket(#path, request.into_request()).await
+        pub async fn #ident<Req>(&mut self, req: Req) -> ClientResult<Socket<#request, #response>>
+        where
+            Req: IntoRequest<()>,
+        {
+            self.inner.connect_socket(#path, req.into_request()).await
         }
     }
 }
@@ -124,11 +119,11 @@ fn generate_server_streaming<T: Method>(method: &T, proto_path: &str, path: Stri
     let (request, response) = method.request_response_name(proto_path);
 
     quote! {
-        pub async fn #ident(
-            &mut self,
-            request: impl IntoRequest<#request>,
-        ) -> ClientResult<Socket<#request, #response>> {
-            self.inner.connect_socket_req(#path, request.into_request()).await
+        pub async fn #ident<Req>(&mut self, req: Req) -> ClientResult<Socket<#request, #response>>
+        where
+            Req: IntoRequest<#request>,
+        {
+            self.inner.connect_socket_req(#path, req.into_request()).await
         }
     }
 }
